package jdomain.jdraw;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import jdomain.util.Log;
import jdomain.util.Util;

/**
 * @author J-Domain [Michaela Behling]
 */
public final class Settings implements Serializable {

   private static final long serialVersionUID = 0L;
   public static final Settings INSTANCE = createSettings();

   private float jpegQuality = 1.0f;

   private Settings() {
   }

   private static String getFilename() {
      StringBuffer buf = new StringBuffer();
      buf.append( System.getProperty( "user.home" ) );
      buf.append( File.separator ).append( "jdraw.dat" );
      return buf.toString();
   }

   public float getJPEGQuality() {
      return jpegQuality;
   }

   public void setJPEGQuality( float quality ) {
      jpegQuality = quality;
   }

   private static Settings createSettings() {
      Log.debug( "loading settings..." );
      File file = new File( getFilename() );
      if ( file.exists() ) {
         ObjectInputStream in = null;
         try {
            in = new ObjectInputStream( new FileInputStream( file ) );
            Settings settings = (Settings)in.readObject();
            settings.check();
            return settings;
         }
         catch ( Exception e ) { // ignore
         }
         finally {
            Util.close( in );
         }
      }
      Settings settings = new Settings();
      settings.check();
      return settings;
   }

   private void check() {

   }

   public void save() {
      Log.debug( "saving settings..." );
      ObjectOutputStream out = null;
      try {
         out = new ObjectOutputStream( new FileOutputStream( getFilename() ) );
         out.writeObject( this );
      }
      catch ( Exception e ) {
         Log.error( "saving settings failed" );
      }
      finally {
         Util.close( out );
      }
   }
}