package jdomain.jdraw.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Hashtable;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import jdomain.jdraw.Settings;
import jdomain.util.Util;
import jdomain.util.gui.WidgetFactory;

/*
 * SettingsDialog - created on 15.12.2005
 * 
 * @author Michaela Behling
 */

public class SettingsDialog extends DrawDialog implements DocumentListener, ChangeListener {

   /** */
   private static final long serialVersionUID = 1L;
   private final JSlider qualitySlider = createSlider();
   private final JTextField qualityField = new JTextField( 4 );
   private boolean sliderChanged;
   private boolean fieldChanged;

   public SettingsDialog() {
      super( "Settings" );
      setUndecorated( false );
      WidgetFactory.addFocusAdapter( qualityField );

      setDefaultBorder();
      setModal( true );
      JPanel p = new JPanel( new GridBagLayout() );
      GridBagConstraints gc = new GridBagConstraints();

      qualitySlider.setValue( (int)(Settings.INSTANCE.getJPEGQuality() * 100) );
      qualityField.setText( String.valueOf( qualitySlider.getValue() ) );
      qualitySlider.addChangeListener( this );
      qualityField.getDocument().addDocumentListener( this );

      gc.gridx = 0;
      gc.gridy = 0;
      gc.fill = GridBagConstraints.HORIZONTAL;
      gc.insets = new Insets( 0, 2, 2, 2 );

      gc.gridwidth = 1;
      JLabel l = new JLabel( "JPEG Quality " );
      p.add( l, gc );

      gc.gridx++;
      p.add( qualitySlider, gc );
      gc.gridx++;
      p.add( qualityField, gc );

      main.add( p, BorderLayout.CENTER );

      addRightButton( getApproveButton() );
      getRootPane().setDefaultButton( getApproveButton() );

      addRightButton( getCancelButton() );
      addButtonPanel();

      setFirstFocusComponent( qualityField );
   }

   public float getQuality() {
      float quality = qualitySlider.getValue() / 100f;
      return quality;
   }

   private JLabel createLabel( String text, int value ) {
      JLabel label = new JLabel();
      label.setFont( MainFrame.TINY_FONT );
      if ( text == null ) {
         label.setText( String.valueOf( value ) );
      }
      else {
         label.setText( text );
      }
      return label;
   }

   private JSlider createSlider() {
      JSlider slider = new JSlider( JSlider.HORIZONTAL, 0, 100, 100 );

      Hashtable map = new Hashtable();
      slider.setMajorTickSpacing( 10 );
      slider.setMinorTickSpacing( 5 );
      slider.setPaintTicks( true );
      map.put( new Integer( 0 ), createLabel( "Lowest", 0 ) );
      map.put( new Integer( 50 ), createLabel( "Medium", 50 ) );
      map.put( new Integer( 100 ), createLabel( "Highest", 100 ) );
      slider.setLabelTable( map );
      slider.setPaintLabels( true );
      return slider;
   }

   private void checkInput() {
      String p = qualityField.getText().trim();
      if ( !Util.isNumber( p ) ) {
         getApproveButton().setEnabled( false );
      }
      else {
         getApproveButton().setEnabled( Util.isIn( Util.asInt( p ), 0, 100 ) );
         if ( !sliderChanged ) {
            if ( getApproveButton().isEnabled() ) {
               fieldChanged = true;
               qualitySlider.setValue( Util.asInt( p ) );
               fieldChanged = false;
            }
         }
      }
   }

   public void changedUpdate( DocumentEvent e ) {
      checkInput();
   }

   public void insertUpdate( DocumentEvent e ) {
      checkInput();
   }

   public void removeUpdate( DocumentEvent e ) {
      checkInput();
   }

   public void stateChanged( ChangeEvent e ) {
      if ( fieldChanged ) {
         return;
      }
      sliderChanged = true;
      qualityField.setText( String.valueOf( qualitySlider.getValue() ) );
      sliderChanged = false;
   }

}
